@@ -75,6 +75,9 @@ module Parser
       alias on_mlhs     process_regular_node
       alias on_masgn    process_regular_node
 
+      alias on_rasgn    process_regular_node
+      alias on_mrasgn   process_regular_node
+
       def on_const(node)
         scope_node, name = *node
 
@@ -124,6 +127,7 @@ module Parser
       alias on_kwarg          process_argument_node
       alias on_kwoptarg       process_argument_node
       alias on_kwrestarg      process_argument_node
+      alias on_forward_arg    process_argument_node
 
       def on_procarg0(node)
         if node.children[0].is_a?(Symbol)
@@ -159,6 +163,8 @@ module Parser
         ])
       end
 
+      alias on_def_e on_def
+
       def on_defs(node)
         definee_node, name, args_node, body_node = *node
 
@@ -168,6 +174,8 @@ module Parser
         ])
       end
 
+      alias on_defs_e on_defs
+
       alias on_undef    process_regular_node
       alias on_alias    process_regular_node
 
@@ -250,6 +258,7 @@ module Parser
       alias on_array_pattern_with_tail process_regular_node
       alias on_hash_pattern            process_regular_node
       alias on_const_pattern           process_regular_node
+      alias on_find_pattern            process_regular_node
 
       # @private
       def process_variable_node(node)
@@ -274,6 +283,10 @@ module Parser
           'Parser::AST::Processor#on_argument instead.'
         on_argument(node)
       end
+
+      def on_empty_else(node)
+        node
+      end
     end
   end
 end
