@@ -26,6 +26,6 @@ class TestBase < Minitest::Test
   def test_loc_dup
     ast = Parser::CurrentRuby.parse('1')
     assert_nil ast.loc.dup.node
-    Parser::AST::Node.new(:root, [], :location => ast.loc)
+    Parser::AST::Node.new(:zsuper, [], :location => ast.loc)
   end
 end
